require 'uu/os/logger'

module UU
  module OS
    module Application
      # @private
      class Logger

        def initialize(name = nil)
          @name = name
          @name = self.class if @name.nil?
        end

        def info(message, &block)
          logger do |log|
            log.info(message)
          end
        end

        def debug(message, &block)
          logger do |log|
            log.debug(message)
          end
        end

        def warn(message, &block)
          logger do |log|
            log.warning(message)
          end
        end

        def error(message, &block)
          logger do |log|
            log.error(message)
          end
        end

        def fatal(message, &block)
          logger do |log|
            log.error(message)
          end
        end

        private

        def logger(&block)

          return unless block_given?

          do_logout = false
          # In case we need to log error and nobody is authenticated, try to login as
          # default application identity (we need authenticated user for using uuAppLog).
          if !UU::OS::Security::Session.logged_in?
            begin
              credentials = UU::OS::Application::Config[:app_credentials]
              unless credentials.nil?
                UU::OS::Security::Session.login(*credentials)
                do_logout = true
              end
            rescue Exception => e
              # Authentication failed, loggedge message will not be available in uuAppLog
              log = UU::OS::Logger.new(self.class)
              log.error("Unable to use uuAppLog. Nobody authenticated.", e)
            end
          end
          @logger ||= UU::OS::Logger.new(@name)
          yield(@logger)
        ensure
          if do_logout
            UU::OS::Security::Session.logout
          end

        end

      end
    end
  end
end
