module UU
  module OS
    module Application
      class UseCaseContext

        # URI of main object (object to be processed by invoked command).
        # @return [String] URI of main object.
        attr_reader :uu_uri

        # Use case parameters.
        # @return [Hash] Use case parameters.
        attr_reader :parameters

        # Session object.
        # @return [UU::OS::Security::Session] Session object.
        attr_reader :session

        # Instance of use case request. Note that request body
        # might not be available (once use case parameters are
        # parsed).
        # @return [UU::OS::HTTP::HTTPRequest] Instance of request.
        # @private
        attr_reader :request # :nodoc:


        # Creates new instance of execution context.
        # @param [Hash] data Initialization data:
        # @option data [URI, String] :uu_uri URI of main object.
        # @option data [Hash] :parameters Use case parameters.
        # @option data [UU::OS::Security::Session] :session Instance of session object.
        # @option data [UU::OS::HTTP::HTTPRequest] :request Instance of request.
        # it means that invocation of invoke_async and invoke_async_status ends up with exception.
        # @private
        def initialize(data = {}) # :nodoc:
          @uu_uri = data[:uu_uri]
          @parameters = data[:parameters]
          @session = data[:session]
          @request = data[:request]
        end

        # Override inspect to hide internal structures.
        # @private
        def inspect # :nodoc:
          "#<#{self.class.name}:0x#{(self.object_id << 1).to_s(16)}>"
        end

      end
    end
  end
end








