require 'uu/os/server/abstract_middleware'
require 'uu/os/application/use_case_context'
require 'uu/os/security/session'
require 'uu/os/server/request'
require 'uu/os/lang/uniform_hash'

module UU
  module OS
    module Application
      class UseCaseContextMiddleware < UU::OS::Server::AbstractMiddleware

        def initialize(app)
          super(app)
        end

        def call(env)
          request = UU::OS::Server::Request.new(env)
          context = get_context(request)
          unless context.nil?
            env['uu.context'] = context
          end
          @app.call(env)
        end

        # @private
        def get_context(request)
          data = prepare_data(request)
          UU::OS::Application::UseCaseContext.new(data)
        end

        # @private
        def prepare_data(request)
          parameters = UU::OS::Lang::UniformHash.new
          parameters.replace(request.params)

          data = {}
          data[:request] = request
          data[:uu_uri] = parameters.delete(main_object_uri_parameter_name)
          data[:parameters] = parameters
          data[:session] = UU::OS::Security::Session.current_session

          ctx = parameters.delete(context_parameter_name)
          data[:uuCtx] = ctx
          set_uu_uri_to_logger_context(data[:uu_uri])
          return data
        end


        # @private
        def set_uu_uri_to_logger_context(uu_uri)
          return unless uu_uri
          UU::OS::Logger::MDCProvider.put('uu-entityURI', uu_uri)
        end

        # Returns name of main object URI parameter (defaults to +:uuUri+).
        # @return [Symbol] Main object URI parameter name.
        # @note Not intended to be used from controller implementation but for
        #    optional overriding of default behavior by custom implementation!
        # @private
        def main_object_uri_parameter_name # :nodoc:
          return :uuUri
        end

        # Returns name of parameter which is used for wrapping of invocation
        # context parameters (defaults to +:uuCtx+).
        # @return [Symbol] Context parameter name.
        # @note Not intended to be used from controller implementation but for
        #    optional overriding of default behavior by custom implementation!
        # @private
        def context_parameter_name # :nodoc:
          return :uuCtx
        end

      end
    end
  end
end
